/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************  
 * Draw curves interactively by clicking in the display window
 *****************************************************************************/


IntList xCoords;  // dynamic list of the point's x-coordinate
IntList yCoords;  // dynamic list of the point's y-coordinate
int[] xCopy;      // copy of the x coordinates required for animation
int[] yCopy;      // copy of the y coordinates required for animation

boolean enableAnimation = false; // flag if animation is enabled
int mode = 0;                    // mode index: 0 = lines, 1 = curves, 2 = Bezier curves


void setup() {
  size(500, 500); 
  
  // create list objects
  xCoords = new IntList();
  yCoords = new IntList();
  
  ellipseMode(CENTER);
  
  curveTightness(0.0);
}

void draw() {
  background(0);
  
  // copy x and y coordinates to be changed by animation function
  xCopy = xCoords.array();
  yCopy = yCoords.array();
  
  if (enableAnimation)
    animate();
  
  if (mode == 0) {
    drawLines();
  }
  else if (mode == 1) {
    drawCurves();
  }
  else if (mode == 2) {
    drawBeziers();
  }
  
  drawPoints();
}


// rotates curve points around the position of the clicked points 
void animate() {
  
  for (int i = 0; i < xCoords.size(); i++) {
    xCopy[i] += (int)( 20*sin(millis()/1000.0 + i*PI));
    yCopy[i] += (int)( 20*cos(millis()/1000.0 - (i*PI % 4)));
  }
  
}

// draw straight lines between points
void drawLines() {

  noFill();
  stroke(255);
  beginShape();
  
  for (int i = 0; i < xCoords.size(); i++) {
    vertex(xCopy[i], yCopy[i]);
  }
  
  endShape();
}

// draw curves
void drawCurves() {

  noFill();
  stroke(255);
  beginShape();
  
  for (int i = 0; i < xCoords.size(); i++) {
    curveVertex(xCopy[i], yCopy[i]);
  }
  
  endShape();
}

// draw Bezier curves
void drawBeziers() {

  noFill();
  stroke(255);
  beginShape();
  
  if (xCoords.size() > 2) {
    vertex(xCopy[0], yCopy[0]);
    
    for (int i = 1; i < xCoords.size()-1; i += 3) {
      bezierVertex(xCopy[i], yCopy[i],
                   xCopy[i+1], yCopy[i+1],
                   xCopy[i-1], yCopy[i-1]);
    }
  }
  
  endShape();
}


// draw curve points from red (first point) to green (last point)
void drawPoints() {
  int n = xCoords.size();
  noFill();
  
  for (int i = 0; i < n; i++) {
    if (n > 1)
      stroke(255*(n-1-i)/(float)(n-1), 255*i/(float)(n-1), 0);
    else
      stroke(0, 255, 0);
      
    ellipse(xCopy[i], yCopy[i], 10, 10);
  }
  
  endShape();
}


// mouse click handler. Adds point on left mouse click, removes last point on right mouse click
void mouseClicked() {
  
  if (mouseButton == LEFT) {
    xCoords.append(mouseX);
    yCoords.append(mouseY);
  }
  else if (mouseButton == RIGHT && xCoords.size() > 0) {
    xCoords.remove(xCoords.size()-1);
    yCoords.remove(yCoords.size()-1);
  }
}

// key handler. Cycles through the 3 modes on 'M' pressed and enables/disables animation on 'A' pressed
void keyPressed() {

  if (key == 'm') {
    mode = (mode + 1) % 3;
  }
  else if (key == 'a') {
    enableAnimation = !enableAnimation; 
  }
}
